#ifndef compiler_lexer_lexer_h
#define compiler_lexer_lexer_h

#include "../streams/stream_input.h"
#include <string>

namespace lexer
{
	//
	// Token
	class token
	{
	public:

		// types
		enum tp
		{
			null = 0,

			// Utility simbols
			open_bracket,  // (
			close_bracket, // )
			quote,         // '
			point,         // .

			// data types
			character,     // #\c , #\space #\newline
			string,        // "...."
			numeric,       // int, rational, real, complex
			boolean,       // #t, #f

			// keywords
			lambda,        // lambda
			if_,           // if
			cond,          // cond
			and_,          // and
			or_,           // or
			define,        // define
			else_,         // else

			variable       // +,-,<variable>
		};

		// set of constructors
		token();
		token(tp t);
		token(tp t, const std::string& str);
		token(double value);
		token(bool value);
		token(char value);

		token(const token& t);
		token& operator=(const token& t);

		// classification
		tp type() const;

		bool isLiteral() const;
		bool isIdentifier() const;
		bool isVariable() const;

		// value
		const std::string& getString() const;
		double getNumeric() const;
		bool getBoolean() const;
		char getCharacter() const;

	private:

		tp m_type;

		std::string m_str;
		double m_numeric;
		bool m_boolean;
		char m_character;
	};

	//
	// Token source. 
	// Read stream of characters, parse it and generate stream of tokens 
	class token_source : public stream::source<token>
	{
	public:
	
		token_source(stream::source<char>& i);
		virtual token get();

	private:

		token getMultiSymbolToken(char first);
		token getString();
		std::string getSolidString(char first);	
		void skipComment();

	private:

		stream::input<char> m_input;
	};
};

#endif
