#ifndef compiler_linker_linker_object_h
#define compiler_linker_linker_object_h

#include <vector>
#include <string>
#include "../../__common/binaries.h"

namespace linker
{
	//
	// Some object.
	// May be code or literal
	class object
	{
	public:

		//
		// Creation functions
		//

		// Variable
		static object* create(const std::string& name, binary::link l);

		// Literals
		static object* create(double d);
		static object* create(char c);
		static object* create(bool b);
		static object* create_string(const std::string& str);
		static object* create_symbol(const std::string& str);
		static object* create_pair(int estimate, binary::link car, binary::link cdr);
		static object* create_empty_list();

		// Code
		static object* create_lambda(size_t count, bool list, binary::link exp);
		static object* create_call(int estimate, size_t count, const binary::link* ops);
		static object* create_if(int estimate, binary::link cond, binary::link cons, binary::link alt);
		static object* create_cond(int estimate, size_t count, const binary::link* tests, const binary::link* exps);
		static object* create_or(int estimate, size_t count, const binary::link* tests);
		static object* create_and(int estimate, size_t count, const binary::link* tests);

		//
		// Members
		//

		short flags() const;
		short type() const;
		int estimate() const;

		int size() const;
		const char* get() const;

		// For debug purpose
		const std::string& name() const;
		bool named() const;

	private:

		object(int t, unsigned int e, const void* c, size_t s);
		object(int t, unsigned int e, const std::string& name, const void* c, size_t s);

		static object* create_countAndLinks(int type, int estimate, size_t count, const binary::link* ops);

	private:

		const int m_type;
		const bool m_named;
		unsigned int m_estimate;
		const std::string m_name;
		std::vector<char> m_data;
	};
};

#endif
