#include "standard_functions.h"
#include "../../__common/standard_functions_codes.h"

namespace
{
	class expression_stdfunc_calculate_all : public expression::base
	{
	public:

		expression_stdfunc_calculate_all(int item)
		{
			m_link.l.item = item;
			m_link.l.table = binary::link::standard;
		}

		virtual binary::link link(linker::context& c)
		{
			return m_link;
		}

		virtual int estimateApply(linker::context& c, size_t argc, base** args)
		{
			int est = base::estimate(c);

			for (size_t i = 0; i < argc; i++)
			{
				est += args[i]->estimate(c);
			}

			return est;
		}

	private:

		binary::link m_link;
	};

	class expression_stdfunc_no_calculate : public expression::base
	{
	public:

		expression_stdfunc_no_calculate(int item)
		{
			m_link.l.item = item;
			m_link.l.table = binary::link::standard;
		}

		virtual binary::link link(linker::context& c)
		{
			return m_link;
		}

		virtual int estimateApply(linker::context& c, size_t argc, base** args)
		{
			return (int)argc + base::estimate(c);;
		}

	private:

		binary::link m_link;
	};

	class expression_stdfunc_calculate_first : public expression::base
	{
	public:

		expression_stdfunc_calculate_first(int item)
		{
			m_link.l.item = item;
			m_link.l.table = binary::link::standard;
		}

		virtual binary::link link(linker::context& c)
		{
			return m_link;
		}

		virtual int estimateApply(linker::context& c, size_t argc, base** args)
		{
			int est = base::estimate(c);

			if (argc >= 1)
			{
				est += args[0]->estimate(c);
			}

			return est;
		}

	private:

		binary::link m_link;
	};
}

const name2expr& linker::getStandardExprs()
{
	static name2expr map;
	static bool inited = false;

	if (!inited)
	{
		map.add("=", new expression_stdfunc_calculate_all(binary::stdfunc::equivalence));
		map.add("eqv?", new expression_stdfunc_calculate_all(binary::stdfunc::equivalence));

		map.add("+", new expression_stdfunc_calculate_all(binary::stdfunc::plus));
		map.add("-", new expression_stdfunc_calculate_all(binary::stdfunc::minus));
		map.add("*", new expression_stdfunc_calculate_all(binary::stdfunc::multiplication));
		map.add("/", new expression_stdfunc_calculate_all(binary::stdfunc::division));
		map.add("sin", new expression_stdfunc_calculate_all(binary::stdfunc::sine));
		map.add("cos", new expression_stdfunc_calculate_all(binary::stdfunc::cosine));
		map.add("tan", new expression_stdfunc_calculate_all(binary::stdfunc::tangent));
		map.add("asin", new expression_stdfunc_calculate_all(binary::stdfunc::arcsine));
		map.add("acos", new expression_stdfunc_calculate_all(binary::stdfunc::arccosine));
		map.add("atan", new expression_stdfunc_calculate_all(binary::stdfunc::arctangent));
		map.add("exp", new expression_stdfunc_calculate_all(binary::stdfunc::exponent));
		map.add("log", new expression_stdfunc_calculate_all(binary::stdfunc::logarithm));

		map.add("car", new expression_stdfunc_calculate_all(binary::stdfunc::car));
		map.add("cdr", new expression_stdfunc_calculate_all(binary::stdfunc::cdr));
		map.add("cons", new expression_stdfunc_no_calculate(binary::stdfunc::cons));
		map.add("list", new expression_stdfunc_no_calculate(binary::stdfunc::list));
		map.add("length", new expression_stdfunc_calculate_all(binary::stdfunc::length));
		map.add("map", new expression_stdfunc_calculate_first(binary::stdfunc::map));

		map.add("null?", new expression_stdfunc_calculate_all(binary::stdfunc::isNull));
		map.add("real?", new expression_stdfunc_calculate_all(binary::stdfunc::isReal));
		map.add("pair?", new expression_stdfunc_calculate_all(binary::stdfunc::isPair));
	}

	return map;
}
