#include "./streams/char_source.h"
#include "./lexer/lexer.h"
#include "./syntacter/syntacter.h"
#include "./linker/linker.h"

#include "./streams/stream_output.h"
#include "../__common/binaries.h"

#include <fstream>
#include "./utils/file.h"

#include <stdexcept>

int main(int argc, char* argv[])
{
	if (argc != 3)
	{
		printf("Usage: compiler.exe <inputfile> <outputfile>\n");
		return 0;
	}

	std::ifstream input;
	input.open(argv[1], std::ios_base::in);

	try
	{
		//   
		char_source cs(input);
		//     
		lexer::token_source ts(cs);
		//   
		stream::input<lexer::token> tokens(ts);
		//  
		syntacter::program syn_prg(tokens);
		// 
		linker::program prg(syn_prg);

		stream::output o;

		binary::header h;
		h.signature = binary::signature;
		h.version = binary::version;
		h.objectTableOffset = (unsigned int)sizeof(h);
		h.objectTableBytesSize = (unsigned int)(prg.size() * sizeof(binary::record));

		o.write_back(&h, sizeof(h));

		int offsetBase = h.objectTableOffset + h.objectTableBytesSize;
		int offset = offsetBase;

		{for (size_t i = 0; i < prg.size(); i++)
		{
			binary::record rec;
			rec.flags = prg[i].flags();
			rec.type = prg[i].type();
			rec.estimate = prg[i].estimate();
			rec.size = prg[i].size();
			rec.offset = offset;

			offset += rec.size;

			o.write_back(&rec, sizeof(rec));
		}}

		{for (size_t i = 0; i < prg.size(); i++)
		{
			o.write_back(prg[i].get(), prg[i].size());
		}}

		CFile f;

		if (!f.create(argv[2]))
		{
			throw std::runtime_error("compiler:main: can not open output file");
		}

		f.write(o.buffer(), o.size());
	}
	catch (stream::eof& )
	{
	}
	catch (std::exception& e)
	{
		fprintf(stderr, "%s", e.what());
	}
}
