#include "syntacter.h"

#include <stdexcept>

/////////////////////////////////////////
//
// Program
// 
/////////////////////////////////////////

syntacter::program::program(stream::input<lexer::token>& input) : m_exp(0)
{
	for (;;)
	{
		lexer::token t = input.get();

		if (t.type() != lexer::token::open_bracket)
		{
			// expresiion
			input.putback(t);
			m_exp = read(input);
			break;
		}

		lexer::token t2 = input.get();

		if (t2.type() == lexer::token::define)
		{
			// definition

			// name
			lexer::token name = input.get();

			if (name.type() != lexer::token::variable)
			{
				throw std::runtime_error("syntacter::program: waiting for variable");
			}

			// expression
			std::auto_ptr<expression::base> e(read(input));

			// )
			lexer::token bracket = input.get();

			if (bracket.type() != lexer::token::close_bracket)
			{
				throw std::runtime_error("syntacter::program: waiting for )");
			}

			m_defs.add(name.getString(), e.release());
		}
		else
		{
			// expresiion
			input.putback(t2);
			input.putback(t);
			m_exp = read(input);
			break;
		}
	}
}

syntacter::program::~program()
{
	delete m_exp;
}

syntacter::definitions& syntacter::program::defs()
{
	return m_defs;
}

expression::base* syntacter::program::exp()
{
	return m_exp;
}

/////////////////////////////////////////
//
// Definitions
// 
/////////////////////////////////////////

size_t syntacter::definitions::size() const
{
	return m_names.size();
}

const std::string& syntacter::definitions::name(size_t i) const
{
	return m_names[i];
}

expression::base* syntacter::definitions::exp(size_t i)
{
	return m_exps[i];
}

void syntacter::definitions::add(const std::string& name, expression::base* e)
{
	m_names.push_back(name);
	m_exps.push_back(e);
}

int syntacter::definitions::getNameIndex(const std::string& name) const
{
	for (size_t i = 0; i < m_names.size(); i++)
	{
		if (name == m_names[i]) return (int)i;
	}

	return -1;
}
