#include "context.h"
#include "../object/object_stdfunc.h"
#include "../../__common/binaries.h"

#include <stdexcept>

///////////////////////////
//
// global program context
//
///////////////////////////

context::global::global(const loader::executable& exe) : m_exe(exe)
{
	addref();
}

object::proxy context::global::get(binary::link l, bool separable) const
{
	return get(l, separable, this);
}

object::proxy context::global::get(binary::link l, bool separable, const base* c) const
{
	if (l.l.table == binary::link::local)
	{
		// use local context instead
		throw std::runtime_error("context::global::get: try to get local link");
	}

	if (l.l.table == binary::link::program)
	{
		// it is a program link
		if (l.l.item >= m_exe.size())
		{
			throw std::runtime_error("context::global::get: l.l.item >= m_exe.size()");
		}

		// create object, set c as it context
		return object::create(m_exe[l.l.item], separable, c);
	}

	if (l.l.table == binary::link::standard)
	{
		// link to standard function

		// Create
		return object::stdfunc::create(l);
	}

	throw std::runtime_error("context::global::get: unknown table type");
}

const context::base* context::global::parent() const
{
	return this;
}

///////////////////////////
//
// local function context
//
///////////////////////////

context::local::local(const base* c) : m_parent(c->parent())
{
	m_local.reserve(8);
}

context::local::~local() 
{
}

void context::local::add(object::proxy p)
{
	m_local.push_back(p);
}

object::proxy context::local::get(binary::link l, bool separable) const
{
	return get(l, separable, this);
}

object::proxy context::local::get(binary::link l, bool separable, const base* c) const
{
	if (l.l.table == binary::link::local)
	{
		// use local context
		if (l.l.item >= m_local.size())
		{
			throw std::runtime_error("context::local::get: to big argument index");
		}

		return m_local[l.l.item];
	}

	return m_parent->get(l, separable, c);
}

const context::base* context::local::parent() const
{
	return m_parent.get();
}
