#include "./loader/loader.h"
#include "./object/object_base.h"
#include "./context/context.h"
#include "./object/object_literals.h"
#include "./tasks/pool.h"

#include <stdexcept>
#include "./utils/timer.h"

int main(int argc, char* argv[])
{
	if (argc < 2)
	{
		printf("Usage: executer.exe <file> [<pool threads count>]\n");
		return 0;
	}

	try
	{
		timer tmp;

		// load file
		loader::executable exe(argv[1]);
		// context
		context::global gc(exe);

		if (exe.size() == 0)
		{
			return 0;
		}

		if (argc == 3)
		{
			// initialize pool 
			tasks::getPool().init(atoi(argv[2]));
		}

		// last entry in objects table - main expression
		object::proxy p = object::create(
							exe[exe.size() - 1], 
							(tasks::getPool().size() > 0), 
							&gc);
		
		// finish evaluation
		p = p.simplify();

		// just print value
		printf("%s\n", p->toString().c_str());

		printf("time: %d milliseconds\n", tmp.get());

		return 0;
	}
	catch (std::runtime_error& e)
	{
		printf("Exception: %s\n", e.what());
	}

	return 0;
}