#ifndef executer_object_object_base_h
#define executer_object_object_base_h

#include "../loader/loader.h"
#include "../context/context_base.h"
#include "../utils/allocator.h"

#include <string>
#include <stdexcept>

namespace object
{
	class proxy;

	namespace types
	{
		enum
		{
			numeric = 0,
			boolean,
			string,
			character,
			symbol,
			pair,
			empty_list,
			function,

			other
		};
	}

	//
	// Create some object, return it proxy
	proxy create(const loader::record& rec, bool separable, const context::base* c);

	//
	// Abstract object
	class base : protected refobject
	{
	public:

		int type() const
		{
			return m_type;
		}

		bool isSimple() const
		{
			return m_simple;
		}

		//
		// common functions
		//

		virtual bool equiv(const base* o) const;
		virtual bool isTrue() const;
		
		bool isNull() const
		{
			return (m_type == types::empty_list);
		}

		bool isNumeric() const
		{
			return (m_type == types::numeric);
		}
		
		bool isPair() const
		{
			return (m_type == types::pair);
		}

		//
		// estimate
		//

		bool isRecursive() const
		{
			return m_recursive;
		}

		//
		// evaluation
		//

		virtual proxy evaluate(bool separable) const;
		virtual proxy apply(bool separable, size_t argc, const proxy* args) const;

		//
		// print value
		// 

		virtual std::string toString() const = 0;

	protected:

		base(int t);
		base(int t, const context::base* c, int est, bool simple = false);

		virtual ~base();

		friend class smartptr<base>;

	private:

		const int m_type;
		bool m_recursive;

	protected:
		
		// can be changed while evaluation - pair
		mutable bool m_simple;
		mutable smartptr<context::base> m_c;
	};

	//
	// Object proxy
	class proxy
	{
	public:

		proxy();
		proxy(const base* p);
		proxy(const base* p, bool separable);

		proxy(const proxy& p) : m_p(p.m_p), m_separable(p.m_separable), m_id(p.m_id)
		{
		}

		proxy& operator=(const proxy& p)
		{
			if (&p != this)
			{
				m_p = p.m_p;
				m_separable = p.m_separable;
				m_id = p.m_id;
			}

			return *this;
		}

		~proxy()
		{			
		}

		const base* operator->() const
		{
			return m_p.get();
		}

		const base* get() const
		{
			return m_p.get();
		}

		template<typename T, int i>
		const T* get() const
		{
			if (m_p->type() == i)
			{
				return (const T*)(m_p.get());
			}

			throw std::runtime_error("object::proxy::get: wrong cast");
		}

		proxy simplify() const;

	private:

		bool m_separable;
		smartptr<base> m_p;
		int m_id;
	};
}

#endif
