#include "object_code.h"
#include "object_literals.h"
#include "../utils/myvector.h"

#include <stdexcept>

// for simplify definitions
using object::proxy;

//////////////////////////	
//
// call object
//
//////////////////////////

//
// creation
//

proxy object::call::create(const context::base* c, const loader::record& rec, bool separable)
{
	if (rec.size() < sizeof(binary::call))
	{
		throw std::runtime_error("object::call::create: wrong size!");
	}

	const binary::call* p = (const binary::call*)(rec.data());

	if (rec.size() != sizeof(binary::call) + (p->argc - 1) * sizeof(binary::link))
	{
		throw std::runtime_error("object::call::create: wrong size!");
	}

	return proxy(new call(c, *p, rec.estimate()), separable);
}

object::call::call(const context::base* ctx, const binary::call& c, int est) : base(object::types::other, ctx, est)
{
	m_func = c.ops[0];
	m_operands.reserve(c.argc);

	for (size_t i = 1; i < c.argc; i++)
	{
		m_operands.push_back(c.ops[i]);
	}
}

//
// overrides
//

proxy object::call::evaluate(bool separable) const
{
	// begin evaluation of function
	proxy func = m_c->get(m_func, separable);

	myvector<proxy> args;
	args.reserve(m_operands.size());

	for (size_t i = 0; i < m_operands.size(); i++)
	{
		// begin evaluation of all arguments
		args.push_back(m_c->get(m_operands[i], separable));
	}

	// evaluate function
	func = func.simplify();

	// and apply it to arguments
	return func->apply(separable, args.size(), args.get());
}

std::string object::call::toString() const
{	
	return "#call";
}

///////////////////////////////
//
// if_ object
//
///////////////////////////////

proxy object::if_::create(const context::base* c, const loader::record& rec, bool separable)
{
	if (rec.size() < sizeof(binary::if_))
	{
		throw std::runtime_error("object::if_::create: wrong size!");
	}

	const binary::if_* p = (const binary::if_*)(rec.data());

	return proxy(new if_(c, *p, rec.estimate()), separable);
}

object::if_::if_(const context::base* ctx, const binary::if_& i, int est) : base(object::types::other, ctx, est)
{
	m_condition = i.condition;
	m_consequence = i.consequence;
	m_alternate = i.alternate;
}

proxy object::if_::evaluate(bool separable) const
{
	proxy cond = m_c->get(m_condition, false).simplify();
	
	if (cond->isTrue())
	{
		return m_c->get(m_consequence, separable);
	}

	return m_c->get(m_alternate, separable);
}

std::string object::if_::toString() const
{
	return "#if";
}