#include "object_lambda.h"
#include "object_literals.h"
#include "../utils/myvector.h"
#include "../context/context.h"

#include <stdexcept>

// for simplify definitions
using object::proxy;
using object::base;

//////////////////////////////////
//
// lambda
//
////////////////////////////////////

proxy object::lambda::create(const context::base* c, const char* v, size_t size)
{
	if (size != sizeof(binary::lambda))
	{
		throw std::runtime_error("object::lambda::create: wrong size!");
	}

	const binary::lambda* p = (const binary::lambda*)v;
	return proxy(new lambda(c, *p));
}

object::lambda::lambda(const context::base* ctx, const binary::lambda& l) 
					: base(object::types::function, ctx, 1, true)
{
	m_argc = l.argc.count;
	m_argList = (l.argc.list != 0);
	m_exp = l.exp;

	if ((m_argc == 0) && m_argList)
	{
		throw std::runtime_error("object::lambda::lambda: (m_argc == 0) && m_argList");
	}
}

proxy object::lambda::apply(bool separable, size_t argc, const proxy* args) const
{
	if ( (!m_argList && (argc != m_argc))
		||(m_argList && (argc < m_argc - 1)))
	{
		throw std::runtime_error("object::lambda::apply: wrong arguments count");
	}

	context::local* plc = new context::local(m_c.get());

	if (!m_argList)
	{
		for (size_t i = 0; i < m_argc; i++)
		{
			plc->add(args[i]);
		}
	}
	else
	{
		// first element is list
		size_t first_in_list = m_argc - 1;

		for (size_t i = 0; i < first_in_list; i++)
		{
			plc->add(args[i]);
		}

		// generate list
		plc->add(pair::list(argc - first_in_list, args + first_in_list));
	}

	return plc->get(m_exp, separable);
}

std::string object::lambda::toString() const
{
	return "#lambda";
}
