#ifndef executer_tasks_pool_h
#define executer_tasks_pool_h

#include "../object/object_base.h"
#include "../utils/smartptr.h"
#include "critical_section.h"
#include "task.h"

#include <vector>
#include <map>

namespace tasks
{
	class pool;

	// Get global pool
	pool& getPool();

	//
	// abstract excecuter of delayed tasks
	class executer
	{
	public:

		executer(pool& p) : m_p(p) {}
		virtual ~executer() {}

	protected:

		pool& m_p;
	};

	//
	// Pool of executers, pending tasks, results.
	class pool
	{
	public:

		void init(size_t count);		
		size_t size() const;

		// Add new delayed task to pool, return task ID
		int push_task(object::proxy p);

		// get result
		object::proxy getResult(int id);

		// make next pending task
		bool makeNextTask();

	private:

		pool()
		{
			m_id = 0;
		}

		~pool();

		friend pool& getPool();

	private:

		std::vector<executer*> m_execs;

		std::vector<int> m_pending;

		typedef std::map<int, smartptr<task> > tasks_pool;
		tasks_pool m_tasks;

		int m_id;

		critical_section m_pendingCs;
		critical_section m_tasksCs;
	};
};

#endif
