#ifndef executer_tasks_task_h
#define executer_tasks_task_h

#include "../utils/allocator.h"
#include "../object/object_base.h"
#include "critical_section.h"
#include "../utils/smartptr.h"

namespace tasks
{
	class task : public refobject
	{
	public:

		task(object::proxy p) : m_p(p.get(), false), m_state(pending)
		{
		}

		bool isReady()
		{
			return (m_state == ready);
		}

		void make()
		{
			locker l(m_cs);

			if (m_state == pending)
			{
				m_state = inprogress;
				m_p = m_p.simplify();
				m_state = ready;
			}
		}

		void tryMake()
		{
			if (m_state != pending)
			{
				return;
			}

			if (m_cs.tryLock())
			{
				unlocker l(m_cs);

				if (m_state == pending)
				{
					m_state = inprogress;
					m_p = m_p.simplify();
					m_state = ready;
				}
			}
		}

		object::proxy get() const
		{
			return m_p;
		}

	private:

		object::proxy m_p;
		critical_section m_cs;

		int m_state;

		enum
		{
			pending = 0,
			inprogress = 1,
			ready = 2
		};
	};
};

#endif
