#ifndef executer_tasks_thread_h
#define executer_tasks_thread_h

#ifdef _WIN32

#ifndef _WINDOWS_
#include <windows.h>
#endif //_WINDOWS_

#else  //_WIN32

#include <pthread.h>
#include <unistd.h>

#endif //_WIN32

//
// Parent for "live" objects
// Windows-Unix portable 
class thread
{
public:

	thread()
	{
		m_hThread = 0;
	}

	~thread()
	{
#ifdef _WIN32
		CloseHandle(m_hThread);
#endif  //_WIN32
	}

	bool start()
	{
#ifdef _WIN32
		m_hThread = CreateThread(0, 0, realThreadFunc, this, 0, 0);
#else  //_WIN32
		pthread_create(&m_hThread, 0, realThreadFunc, this);
#endif //_WIN32
		return (m_hThread != 0);
	}

	void waitForEnd()
	{
#ifdef _WIN32
		WaitForSingleObject(m_hThread, INFINITE);
#else  //_WIN32
		pthread_join(m_hThread, 0);
#endif //_WIN32
	}

protected:

	// Child must override
	virtual long func() = 0;

	// Utility
	void sleep(int ms)
	{
#ifdef _WIN32
		Sleep(ms);
#else  //_WIN32
		::usleep(ms * 1000); // usleep work with mcs
#endif //_WIN32
	}

	void yield()
	{
#ifdef _WIN32
		Sleep(0);
#else  //_WIN32
		pthread_yield();
#endif //_WIN32
	}

private:
	
#ifdef _WIN32
	static DWORD CALLBACK realThreadFunc(void* p)
	{
		thread* t = reinterpret_cast<thread*>(p);
		return t->func();
	}
#else  //_WIN32
	static void* realThreadFunc(void* p)
	{
		thread* t = reinterpret_cast<thread*>(p);
		return (void*)(t->func());
	}
#endif //_WIN32

private:

	// Is not copiable
	thread(const thread&);
	thread& operator=(const thread&);

private:

#ifdef _WIN32
	HANDLE m_hThread;
#else  //_WIN32
	pthread_t m_hThread;
#endif //_WIN32
};

#endif
