#ifndef executer_utils_smartptr_h
#define executer_utils_smartptr_h

#include "allocator.h"

#ifdef USE_FAST_ALLOCATOR

//
// Base class for references object
class refobject
{
public:

	refobject(){}
	virtual ~refobject(){}
	void addref() const{}
	void release() const {}
};

//
// Smart pointer for something
// with methods addref and release
template<typename T>
class smartptr
{
public:

	smartptr(const T* p = 0) : m_p(p)
	{		
	}

	smartptr(const smartptr<T>& p)
	{
		m_p = p.m_p;
	}

	smartptr& operator=(const smartptr<T>& p)
	{
		if (&p != this)
		{
			m_p = p.m_p;
		}

		return *this;
	}

	const T* operator->() const
	{
		return m_p;
	}

	T* operator->()
	{
		return const_cast<T*>(m_p);
	}

	const T* get() const
	{
		return m_p;
	}

	T* get()
	{
		return const_cast<T*>(m_p);
	}

private:

	const T* m_p;
};


#else //use standard allocator

//
// Base class for references object
class refobject
{
public:

	refobject() : m_ref(0)
	{
	}

	virtual ~refobject()
	{
	}

	void addref() const
	{
		m_ref++;
	}

	void release() const 
	{
		m_ref--;
		if (!m_ref) 
		{
			delete this;
		}
	}

private:

	mutable int m_ref;
};

//
// Smart pointer for something
// with methods addref and release
template<typename T>
class smartptr
{
public:

	smartptr(const T* p = 0) : m_p(p)
	{
		if (m_p) m_p->addref();
	}

	smartptr(const smartptr<T>& p)
	{
		m_p = p.m_p;
		if (m_p) m_p->addref();
	}

	smartptr& operator=(const smartptr<T>& p)
	{
		if (&p != this)
		{
			if (m_p) m_p->release();
			m_p = p.m_p;
			if (m_p) m_p->addref();
		}

		return *this;
	}

	~smartptr()
	{
		if (m_p) m_p->release();
	}

	const T* operator->() const
	{
		return m_p;
	}

	T* operator->()
	{
		return const_cast<T*>(m_p);
	}

	const T* get() const
	{
		return m_p;
	}

	T* get()
	{
		return const_cast<T*>(m_p);
	}

private:

	const T* m_p;
};

#endif 

#endif
